@@ -54,65 +54,53 @@ using the configure script.
   */
 
 
-#if 1
-#define NTL_STD_CXX
+#if 0
+#define NTL_LEGACY_NO_NAMESPACE
 
 /* 
- * Use this flag if you want to use the "Standard C++" version of NTL.
- * In this version, all of NTL is "wrapped" inside the namespace NTL,
- * and are no longer directly accessible---you must either use
- * explicit qualification, or using directives, or using declarations.
- * However, note that all names that begin with "NTL_" are macros,
- * and as such do not belong to any namespace.
- * Additionally, instead of including the standard headers
- * <stdlib.h>, <math.h>, and <iostream.h>, the standard headers
- * <cstdlib>, <cmath>, and <iostream> are included.
- * These "wrap" some (but not all) names in namespace std.
- * Also, the 'nothrow' version on the 'new' operator is used.
+ * By default, NTL components are declared inside the namespace NTL.
+ * Set this flag if you want to instead have these components
+ * declared in the global namespace.  This is for backward
+ * compatibility only -- not recommended.
  *
  * To re-build after changing this flag: rm *.o; make ntl.a
  */
 
 #endif
 
-/* The following three flags may be used if you want to use some
- * of the features of Standard C++, but your compiler is deficient.
- * Instead of setting the NTL_STD_CXX, you can set any subset 
- * of the these three.  Setting all three of these flags is equivalent
- * to setting NTL_STD_CXX.  No harm is done if NTL_STD_CXX is set
- * and some of the following three flags are set.
- *
- * To re-build after changing any of these flags: rm *.o; make ntl.a
- */
 
 #if 0
-#define NTL_PSTD_NNS
+#define NTL_LEGACY_INPUT_ERROR
+
+/*
+ * Also for backward compatibility.  Set if you want input 
+ * operations to abort on error, instead of just setting the
+ * "fail bit" of the input stream.
+ *
+ * To re-build after changing this flag: rm *.o; make ntl.a
+ */
 
-/* Set if NTL library components are to be wrapped in namespace 'NTL'. */
 
 #endif
 
 #if 0
-#define NTL_PSTD_NHF
+#define NTL_THREADS
 
-/* Set if you want to use the new header files <cstdlib>, <cmath>, and 
- * <iostream>, instead of the traditional header files <stdlib.h>,
- * <math.h>, and <iostream.h>.
- * If new header files are used, then it is assumed that all standard 
- * library components are wrapped in namespace std; otherwise,
- * it is assumed that all standard library components are in the
- * global namespace.
+/* Set if you want to compile NTL as a thread-safe library.
  *
- * Also, when set, some internal NTL files use the header <fstream>
- * in place of <fstream.h>.                                                      
+ * To re-build after changing this flag: rm *.o; make ntl.a
  */
 
 #endif
 
-#if 0
-#define NTL_PSTD_NTN
 
-/* Set if you want to use the 'nothrow' version of new. */
+#if 0
+#define NTL_EXCEPTIONS
+
+/* Set if you want to compile NTL with exceptions enabled
+ *
+ * To re-build after changing this flag: rm *.o; make ntl.a
+ */
 
 #endif
 
@@ -129,10 +117,7 @@ using the configure script.
  * by the confiuration script when you pass the flag NTL_GMP_LIP=on
  * to that script.
  *
- * Beware that setting this flag can break some older NTL codes.
- * If you want complete backward compatability, but not quite
- * the full performance of GMP, use the flag NTL_GMP_HACK below.
- * See the full NTL documentation for more details.
+ * Beware that setting this flag can break some very old NTL codes.
  *
  * To re-build after changing this flag:
  *   rm *.o; make setup3; make ntl.a
@@ -140,29 +125,6 @@ using the configure script.
  * GMP_OPT_INCDIR, GMP_OPT_LIBDIR, and GMP_OPT_LIB.
  */
 
-#elif 0
-#define NTL_GMP_HACK
-
-/* 
- * Use this flag if you want to use GMP as the long integer package.
- * This can result in significantly faster code on some platforms.
- * It requires that the GMP package (version >= 2.0.2) has already been
- * installed.  You will also have to set the variables GMP_OPT_INCDIR,
- * GMP_OPT_LIBDIR, GMP_OPT_LIB in the makefile (these are set automatically
- * by the confiuration script when you pass the flag NTL_GMP_HACK=on
- * to that script.
- *
- * Unlike the NTL_GMP_LIP flag above, setting this flag maintains
- * complete backward compatability with older NTL codes, but
- * you do not get the full performance of GMP.
- *
- * To re-build after changing this flag:
- *   rm lip.o; make setup3; make ntl.a
- * You may also have to edit the makefile to modify the variables
- * GMP_OPT_INCDIR, GMP_OPT_LIBDIR, and GMP_OPT_LIB.
- *
- */
-
 #endif
 
 #if 0
@@ -223,28 +185,6 @@ using the configure script.
 #endif
 
 
-#if 0
-#define NTL_CXX_ONLY
-
-/*
- *   It is possible to compile everything using C++ only.
- *   If you want to do this, make CC and CXX in the makefile the same.
- *   You may also want to set this flag, which eliminates some
- *   "C" linkage that is no longer necessary.
- *   However, it should still work without it.
- *   
- *   This flag can be set independently of NTL_STD_CXX.
- *   All functions that may have "C" linkage are never wrapped in
- *   namespace NTL;  instead, their names always start with "_ntl_",
- *   and as such, they should not conflict with other global names.
- *   All such names are undocumented, and should never be used 
- *   by NTL clients under normal circumstances.
- *
- *   To re-build after changing this flag: rm *.o; make ntl.a
- */
-
-#endif
-
 #if 0
 #define NTL_CLEAN_INT
 
@@ -343,14 +283,9 @@ using the configure script.
  *************************************************************************/
 
 
-/* One can choose one of four different stragtegies for long integer
- * arithmetic: the default, NTL_LONG_LONG, NTL_AVOID_FLOAT, or NTL_SINGLE_MUL.
- * The configuration wizard will choose among the first three; the use of
- * NTL_SINGLE_MUL is only allowed if NTL_CLEAN_INT is not set, and its
- * use is not recommended.
- *
- * These flags are irrelevant when NTL_GMP_LIP is set, and are simply ignored,
- * except for NTL_SINGLE_MUL -- setting that causes a complie-time error.
+/* One can choose one of three different stragtegies for long integer
+ * arithmetic: the default, NTL_LONG_LONG, or NTL_AVOID_FLOAT.
+ * The configuration wizard will choose among them.
  * 
  */
 
@@ -389,19 +324,6 @@ using the configure script.
  *   To re-build after changing this flag:  rm lip.o; make ntl.a
  */
 
-#elif 0
-#define NTL_SINGLE_MUL 
-
-/*   This was developed originally to improve performance on
- *   ancient Sparc stations that did not have built-in integer mul
- *   instructions.  Unless you have such an old-timer, I would not
- *   recommend using this option.  This option only works on
- *   32-bit machines with IEEE floating point, and is not truly
- *   portable.  If you use this option, you get a 26-bit radix.
- *
- *   To re-build after changing this flag: rm *.o; make ntl.a
- */
-
 #endif
 
 
@@ -411,7 +333,7 @@ using the configure script.
  * and NTL_SPMM_ASM.
  * This plays a crucial role in the  "small prime FFT" used to 
  * implement polynomial arithmetic, and in other CRT-based methods 
- * (such as linear  algebra over ZZ), as well as polynomial andd matrix 
+ * (such as linear  algebra over ZZ), as well as polynomial and matrix 
  * arithmetic over zz_p.  
  */
 
@@ -457,9 +379,50 @@ using the configure script.
 
 
 
+/*
+ * The following two flags provide additional control for how the 
+ * FFT modulo single-precision primes is implemented.
+ */
+
+#if 0
+#define NTL_FFT_BIGTAB
+
+/*
+ * Precomputed tables are used to store all the roots of unity
+ * used in an FFT computation for the first NTL_FFT_BIGTAB_LIMIT
+ * FFT primes (the latter is defined in FFT.h).  This can
+ * lead to significant time savings but at the const of some space:
+ * in the worst case, the precomputed tables will take of space
+ * log_2(NTL_FFT_BUGTAB_LIMIT) * M, where M is roughly the maxmimum
+ * space occupied by any one polynomial that was involved in an
+ * FFT computation (this could be a polynomial over zz_p, ZZ_p, or ZZ).
+ *
+ *   To re-build after changing this flag: rm *.o; make ntl.a
+ */
+
+#endif
 
 
-/* The next five flags NTL_AVOID_BRANCHING, NTL_TBL_REM,
+#if 0
+#define  NTL_FFT_LAZYMUL
+
+/*
+ * This flag only has an effect when combined with the NTL_FFT_BIGTAB
+ * flag, and either the NTL_SPMM_ULL or NTL_SPMM_ASM flags. 
+ * When set, a "lazy multiplication" strategy due to David Harvey:
+ * see his paper "FASTER ARITHMETIC FOR NUMBER-THEORETIC TRANSFORMS".
+ *
+ *   To re-build after changing this flag: rm *.o; make ntl.a
+ */
+
+
+#endif
+
+
+
+
+
+/* The next six flags NTL_AVOID_BRANCHING, NTL_TBL_REM, NTL_TBL_REM_LL,
  * NTL_GF2X_ALTCODE, NTL_GF2X_ALTCODE1, and NTL_GF2X_NOINLINE
  * are also set by the configuration wizard.  
  */
@@ -493,6 +456,20 @@ using the configure script.
  *   ZZ_pX multiplication routines.  If you use the NTL_AVOID_FLOAT 
  *   or NTL_LONG_LONG flags, then you should probably use this one too.
  *
+ *   To re-build after changing this flag: 
+ *      rm lip.o; make ntl.a
+ */
+
+#endif
+
+
+#if 0
+#define NTL_TBL_REM_LL
+
+/*
+ *
+ *   This forces the LONG_LONG version if TBL_REM
+ *
  *   Irrelevent when NTL_GMP_LIP is set.
  *
  *   To re-build after changing this flag: 
@@ -502,6 +479,7 @@ using the configure script.
 #endif
 
 
+
 #if 0
 #define NTL_GF2X_ALTCODE
 
@@ -545,27 +523,6 @@ using the configure script.
 
 
 
-/* The following flag is not set by the configuration wizard;  its use
- * is not generally recommended.
- */
-
- 
-#if 0
-#define NTL_FAST_INT_MUL
-
-/*
- *   Really esoteric.
- *   If using NTL_SINGLE_MUL, and your machine
- *   has a fast integer multiply instruction, this might yield
- *   faster code.  Experiment!
- *
- *   Irrelevent when NTL_GMP_LIP is set.
- *
- *   To re-build after changing this flag: rm *.o; make ntl.a
- */
-
-#endif
-
 
 
 
