@@ -2,44 +2,28 @@
 #ifndef NTL_tools__H
 #define NTL_tools__H
 
+//#define NTL_TEST_EXCEPTIONS
+
 #include <NTL/ctools.h>
+#include <NTL/new.h>
 
-#if (defined(NTL_STD_CXX) || defined(NTL_PSTD_NHF))
-
-// new header files
+#include <iostream>
+#include <new>
+#include <stdexcept>
 
 #include <cstdlib>
 #include <cmath>
-#include <iostream>
 
-#else
 
-// old header files
 
-#include <stdlib.h>
-#include <math.h>
-#include <iostream.h>
-
-#endif
-
-#if (defined(NTL_STD_CXX) || defined(NTL_PSTD_NHF))
 
 #define NTL_SNS std ::
 #define NTL_USE_SNS using namespace std;
 
-#elif (defined(NTL_PSTD_NNS))
+#ifndef NTL_LEGACY_NO_NAMESPACE
 
-#define NTL_SNS ::
-#define NTL_USE_SNS 
-
-#else
-
-#define NTL_SNS 
-#define NTL_USE_SNS 
-
-#endif
-
-#if (defined(NTL_STD_CXX) || defined(NTL_PSTD_NNS))
+// This wraps NTL in the NTL namespace.
+// This is the current default.
 
 #define NTL_NAMESPACE NTL
 #define NTL_OPEN_NNS namespace NTL_NAMESPACE {
@@ -65,6 +49,9 @@
 
 #else
 
+// This puts NTL in the global namespace.
+// Provided only for backward compatibility.
+
 #define NTL_NAMESPACE 
 #define NTL_OPEN_NNS 
 #define NTL_CLOSE_NNS 
@@ -80,29 +67,8 @@
 
 
 
-#if 0
-
-// This is for debugging purposes only.
-
-namespace foo_bar {
-
-class ostream;
-class istream;
-
-typedef unsigned int size_t;
-
-double floor(double);
-float floor(float);
-
-}
-#endif
-
-
-
-#if (!defined(NTL_CXX_ONLY)) 
-extern "C"
-#endif
 double _ntl_GetTime();
+unsigned long _ntl_GetPID();
 
 typedef unsigned long _ntl_ulong;
 typedef _ntl_ulong *_ntl_ulong_ptr;
@@ -113,9 +79,58 @@ typedef _ntl_ulong *_ntl_ulong_ptr;
 
 
 
+
 NTL_OPEN_NNS
 
-extern void (*ErrorCallback)();
+#ifndef NTL_LEGACY_INPUT_ERROR
+
+// this newer version is more in line with wider C++
+// practice, setting the "fail bit" of an input stream
+// when an error is encounted.  This is now the default in NTL
+
+#define NTL_INPUT_ERROR(s, msg) \
+   do {\
+      s.setstate(NTL_SNS ios::failbit);\
+      return s;\
+   } while (0)\
+
+
+#else
+
+// this version provides full backward compatibility,
+// raising an error on ill-formed or missing input
+
+#define NTL_INPUT_ERROR(s, msg) \
+   do {\
+      InputError(msg);\
+   } while (0)\
+
+
+#endif
+
+
+#define NTL_INPUT_CHECK_ERR(stmt) \
+   do {\
+      if (!(stmt)) InputError("bad input\n");\
+   } while (0)\
+
+
+
+#define NTL_INPUT_CHECK_RET(s, stmt) \
+   do {\
+      if (!(stmt)) { s.setstate(NTL_SNS ios::failbit); return s; }\
+   } while (0)\
+
+
+
+
+
+#define NTL_FILE_THRESH (64000.0)
+// threshold in KB for switching to external storage of certain
+// tables (currently in the DDF polynomial factoring routines)
+
+
+
 
 struct INIT_SIZE_STRUCT { };
 const INIT_SIZE_STRUCT INIT_SIZE = INIT_SIZE_STRUCT();
@@ -142,6 +157,19 @@ struct INIT_USER_FFT_STRUCT { };
 const INIT_USER_FFT_STRUCT INIT_USER_FFT = INIT_USER_FFT_STRUCT();
 typedef const INIT_USER_FFT_STRUCT& INIT_USER_FFT_TYPE;
 
+struct INIT_NO_ALLOC_STRUCT { };
+const INIT_NO_ALLOC_STRUCT INIT_NO_ALLOC = INIT_NO_ALLOC_STRUCT();
+typedef const INIT_NO_ALLOC_STRUCT& INIT_NO_ALLOC_TYPE;
+
+struct INIT_ALLOC_STRUCT { };
+const INIT_ALLOC_STRUCT INIT_ALLOC = INIT_ALLOC_STRUCT();
+typedef const INIT_ALLOC_STRUCT& INIT_ALLOC_TYPE;
+
+struct INIT_MONO_STRUCT { };
+const INIT_MONO_STRUCT INIT_MONO = INIT_MONO_STRUCT();
+typedef const INIT_MONO_STRUCT& INIT_MONO_TYPE;
+
+
 
 #ifdef NTL_NO_INIT_TRANS
 #define NTL_OPT_RETURN(t, x) return x
@@ -196,6 +224,12 @@ inline int to_int(unsigned a)
 inline int to_int(unsigned long a) 
    { unsigned y = (unsigned) a;  return NTL_UINT_TO_INT(y); }
 
+/*
+  This function is not present in vanilla NTL 5.4.2.
+  See tools.c for documentation.
+ */
+void SetErrorCallbackFunction(void (*func)(const char *s, void *context), void *context);
+
 
 inline void conv(long& x, int a) { x = a; }
 inline void conv(long& x, long a) { x = a; }
@@ -293,17 +327,11 @@ long CharToIntVal(long c);
 char IntValToChar(long a);
 
 
-/*
-  This function is not present in vanilla NTL 5.4.2.
-  See tools.c for documentation.
- */
-void SetErrorCallbackFunction(void (*func)(const char *s, void *context), void *context);
 
 
-void Error(const char *s);
-
 
 inline double GetTime() { return _ntl_GetTime(); }
+inline unsigned long GetPID() { return _ntl_GetPID(); }
 
 inline long IsFinite(double *p) { return _ntl_IsFinite(p); }
 
@@ -337,6 +365,183 @@ void PrintTime(NTL_SNS ostream& s, double t);
 
 #endif
 
+// A very lightly wrapped pointer than does nothing more than provide
+// auto cleanup in a destructor.  Use the UniquePtr class (in SmartPtr.h) 
+// for a class with more safety and convenience features.
+// This class is easiest to use to retrofit older code with RAII
+// semantics.
+
+// A call to Deleter::apply should free the pointed-to storage
+// and set the pointer itself to zero, so apply should
+// take an argument that is a reference to a T*.
+
+template<class T, class Deleter>
+class WrappedPtr {
+private:
+   WrappedPtr(const WrappedPtr&); // disable
+   void operator=(const WrappedPtr&); // disable
+public:
+   typedef T * raw_ptr;
+
+   raw_ptr rep;
+
+   WrappedPtr() : rep(0) { }
+   void operator=(const raw_ptr& _rep)  { rep = _rep; }
+
+   ~WrappedPtr() { Deleter::apply(rep); } 
+
+   operator const raw_ptr& () const { return rep; }
+   operator raw_ptr& () { return rep; }
+
+   const raw_ptr* operator&() const { return &rep; }
+   raw_ptr* operator&() { return &rep; }
+
+   void kill() { Deleter::apply(rep); }
+
+   void swap(WrappedPtr& other) { _ntl_swap(rep, other.rep); }
+
+};
+
+template<class T, class Deleter>
+void swap(WrappedPtr<T,Deleter>& x, WrappedPtr<T,Deleter>& y)
+{
+   x.swap(y);
+}
+
+
+
+// Error Handling
+
+
+
+class ErrorObject : public NTL_SNS runtime_error {
+public:
+   ErrorObject(const char *msg) : runtime_error(msg) { }
+};
+
+class LogicErrorObject : public ErrorObject {
+public: 
+   LogicErrorObject(const char *msg) : ErrorObject(msg) { }
+};
+
+class ArithmeticErrorObject : public ErrorObject {
+public: 
+   ArithmeticErrorObject(const char *msg) : ErrorObject(msg) { }
+};
+
+class ResourceErrorObject : public ErrorObject {
+public: 
+   ResourceErrorObject(const char *msg) : ErrorObject(msg) { }
+};
+
+class FileErrorObject : public ErrorObject {
+public: 
+   FileErrorObject(const char *msg) : ErrorObject(msg) { }
+};
+
+class InputErrorObject : public ErrorObject {
+public: 
+   InputErrorObject(const char *msg) : ErrorObject(msg) { }
+};
+
+
+
+NTL_THREAD_LOCAL extern void (*ErrorCallback)();
+
+void TerminalError(const char *s);
+
+#ifdef NTL_EXCEPTIONS
+
+inline void MemoryError() { throw NTL_SNS bad_alloc(); }
+inline void Error(const char *msg) { throw ErrorObject(msg); }
+inline void LogicError(const char *msg) { throw LogicErrorObject(msg); }
+inline void ArithmeticError(const char *msg) { throw ArithmeticErrorObject(msg); }
+inline void InvModError(const char *msg) { throw ArithmeticErrorObject(msg); }
+inline void ResourceError(const char *msg) { throw ResourceErrorObject(msg); }
+inline void FileError(const char *msg) { throw FileErrorObject(msg); }
+inline void InputError(const char *msg) { throw InputErrorObject(msg); }
+
+#else
+
+inline void MemoryError() { TerminalError("out of memory"); }
+inline void Error(const char *msg) { TerminalError(msg); }
+inline void LogicError(const char *msg) { TerminalError(msg); }
+inline void ArithmeticError(const char *msg) { TerminalError(msg); }
+inline void InvModError(const char *msg) { TerminalError(msg); }
+inline void ResourceError(const char *msg) { TerminalError(msg); }
+inline void FileError(const char *msg) { TerminalError(msg); }
+inline void InputError(const char *msg) { TerminalError(msg); }
+
+#endif
+
+
+
+
+
+
+#ifdef NTL_EXCEPTIONS
+
+
+template < typename F  >
+class scope_guard 
+{
+    typename std::remove_reference<F>::type f;
+    bool active;
+    const char *info;
+    
+public:
+    scope_guard(F&& _f, const char *_info) : 
+       f(std::forward<F>(_f)), active(true), info(_info) { }
+
+    ~scope_guard() {
+        if (active) {
+#ifdef NTL_TEST_EXCEPTIONS
+            NTL_SNS cerr << "*** ACTIVE SCOPE GUARD TRIGGERED: "
+                         <<  info << "\n";
+#endif
+            f();
+        }
+    }
+
+    void relax() { active = false; }
+};
+
+
+struct scope_guard_builder {  
+   const char *info;
+   scope_guard_builder(const char *_info) : info(_info) { }
+};
+
+template < typename F >
+scope_guard<F> 
+operator+(scope_guard_builder b, F&& f)
+{
+    return scope_guard<F>(std::forward<F>(f), b.info);
+}
+
+
+#define NTL_SCOPE(var) auto var =  \
+   scope_guard_builder(__FILE__ ":" NTL_STRINGIFY(__LINE__)) + [&]
+
+
+#else
+
+
+class DummyScopeGuard {
+public:
+   void relax() { }
+};
+
+#define NTL_SCOPE(var) DummyScopeGuard var; if (false)
+
+
+
+
+#endif
+
+
+
+
 NTL_CLOSE_NNS
 
 
